==> LoadL-3.3.2.8-0.SLES9-PPC64.3.3.X.tar.gz: opt/ibmll/LoadL/full/samples/llbg/bg_bp_query.c <==
// The following code queries and displays information about all base
// partitions in the blue gene system.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"

const char* bpStateString(int);
const char* partitionStateString(int);
const char* nodeCardState(int );
const char* nodeCardQuarter(int );
const char* nodeMode(int );

main(int argc, char *argv[])
{
	LL_element *queryObject;
	LL_element *machine;
	LL_element *bp;
	LL_element  *node_card;
	char *BP_id;
	char *BP_current_partition;
	char *node_card_id;
	char *node_card_current_partition;
	int *BP_size;
	int BP_current_partition_state;
	int BP_node_card_count;
	int node_card_state;
	int node_card_quarter;
	int obj_count;
	int err_code;
	int rc;
	int state;
	int count;
	int boolean;
	int node_card_current_partition_state;
	const char *partid;
	const char *partstate;

	// Initialize the query blue gene
	queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		 exit(1);
	}

	// Set the query request for all base partitions
	char *bp_list[2];
	bp_list[0] = "All";
	bp_list[1] = NULL;

	rc = ll_set_request(queryObject, QUERY_BG_BASE_PARTITION, bp_list, ALL_DATA);
	if (rc) {
		printf("Query MACHINES: ll_set_request() return code = %d\n",
		        rc);
		exit(1);
	}

	// Get the blue gene machine object. Only one machine object is returned
	// The blue gene machine object will only contain data for the base
	// partitions.
	err_code = 0;
	obj_count = 0;
	machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from blue gene machine
	rc = ll_get_data(machine, LL_BgMachineBPSize, &BP_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineBPSize rc = %d\n",rc);
		exit(1);
	}
	printf("Machine BP BP_size = %dx%dx%d\n",BP_size[0], BP_size[1], BP_size[2]);
	free(BP_size);

	//
	// Base Partitions
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstBP, &bp);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstBP rc = %d\n",rc);
		exit(1);
	}

	while(bp != NULL) {
		rc = ll_get_data(bp, LL_BgBPId, &BP_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPLocation, &BP_size);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPLocation rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPSubDividedBusy, &boolean);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPSubDividedBusy rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPCurrentPartition, &BP_current_partition);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPCurrentPartition rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPCurrentPartitionState, &BP_current_partition_state);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPCurrentPartitionState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPNodeCardCount, &BP_node_card_count);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPNodeCardCount rc = %d\n",rc);
			exit(1);
		}

		if (strlen(BP_current_partition) == 0) {
			partid = "NONE";
			partstate = "NONE";
		} else {
			partid = BP_current_partition;
			partstate = partitionStateString(BP_current_partition_state);
		}
		printf("\nBase Partition: id=%s state=%-7.7s loc=%dx%dx%d sdb=%s curPar=%s curPState=%s Node_Card_Count=%d\n",
		      BP_id, bpStateString(state), BP_size[0], BP_size[1], BP_size[2],
		      boolean?"True":"False", partid, partstate, BP_node_card_count);
		free(BP_size);
		free(BP_id);
		free(BP_current_partition);

		//
		// Base Partition's Node Card
		//
		rc = ll_get_data(bp, LL_BgBPGetFirstNodeCard, &node_card);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPGetFirstNodecard rc = %d\n",rc);
			exit(1);
		}

		while(node_card != NULL) {
			rc = ll_get_data(node_card, LL_BgNodeCardId, &node_card_id);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardId rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardState, &node_card_state);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardState rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardQuarter, &node_card_quarter);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardQuarter rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardCurrentPartition, &node_card_current_partition);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardCurrentPartition rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardCurrentPartitionState, &node_card_current_partition_state);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardCurrentPartitionState rc = %d\n",rc);
				exit(1);
			}
			
			if (strlen(node_card_current_partition) == 0) {
				partid = "NONE";
				partstate = "NONE";
			} else {
				partid = node_card_current_partition;
				partstate = partitionStateString(node_card_current_partition_state);
			}
			printf("\tNode Card Id=%s State=%s Quarter=%s curPar=%-13.13s curPState=%s\n",
				node_card_id,nodeCardState(node_card_state),nodeCardQuarter(node_card_quarter),
				partid, partstate);
			free(node_card_id);
			free(node_card_current_partition);

			rc = ll_get_data(bp, LL_BgBPGetNextNodeCard, &node_card);
			if (rc) {
				printf("Error: ll_get_data LL_BgGetNextNodecard rc = %d\n",rc);
				exit(1);
			}

		}

		rc = ll_get_data(machine, LL_BgMachineGetNextBP, &bp);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextBP rc = %d\n",rc);
			exit(1);
		}
	}


	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element 
	ll_deallocate(queryObject);

}

const char* bpStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_BP_UP:
      str_state = "UP";
      break;
    case BG_BP_DOWN:
      str_state = "DOWN";
      break;
    case BG_BP_MISSING:
      str_state = "MISSING";
      break;
    case BG_BP_ERROR:
      str_state = "ERROR";
      break;
    case BG_BP_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* partitionStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_PARTITION_FREE:
      str_state = "FREE";
      break;
    case BG_PARTITION_CONFIGURING:
      str_state = "CFG";  
      break;        
    case BG_PARTITION_READY:
      str_state = "READY";
      break;
    case BG_PARTITION_BUSY:
      str_state = "BUSY";
      break;
    case BG_PARTITION_DEALLOCATING:
      str_state = "DEALC";
      break;
    case BG_PARTITION_ERROR:
      str_state = "ERROR";
      break;
    case BG_PARTITION_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* nodeCardState(int st) {
  char* str_state;
  switch ( st ) {
    case BG_NODE_CARD_UP:
      str_state = "UP";
      break;
    case BG_NODE_CARD_DOWN:
      str_state = "DOWN";
      break;
    case BG_NODE_CARD_MISSING:
      str_state = "MISSING";
      break;
    case BG_NODE_CARD_ERROR:
      str_state = "ERROR";
      break;
    case BG_NODE_CARD_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* nodeCardQuarter(int qt) {
  char* str_quarter;
  switch ( qt ) {
    case BG_QUARTER_Q1:
      str_quarter = "Q1";
      break;
    case BG_QUARTER_Q2:
      str_quarter = "Q2";
      break;
    case BG_QUARTER_Q3:
      str_quarter = "Q3";
      break;
    case BG_QUARTER_Q4:
      str_quarter = "Q4";
      break;
    case BG_QUARTER_Q_NAV:
      str_quarter = "NAV";
      break;
    default:
      str_quarter = "<unknown>";
      break;
  }
  return str_quarter;
}

const char* nodeMode(int m) {
  char* str_mode;
  switch ( m ) {
    case COPROCESSOR:
      str_mode = "COPROCESSOR";
      break;
    case VIRTUAL_NODE:
      str_mode = "VIRTUAL_NODE";
      break;
    default:
      str_mode = "<unknown>";
      break;
  }
  return str_mode;
}

==> LoadL-3.4.0.4-0.SLES9-PPC64.3.4.X.tar.gz: opt/ibmll/LoadL/full/samples/llbg/bg_bp_query.c <==
// The following code queries and displays information about all base
// partitions in the blue gene system.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"

const char* bpStateString(int);
const char* partitionStateString(int);
const char* nodeCardState(int );
const char* nodeCardQuarter(int );
const char* nodeMode(int );


// Convert C-node memory from an enum to an integer in the MB unit
int get_real_memory(int _cnode_memory) {
  int real_memory;
  switch(_cnode_memory) {
  case BG_BP_COMPUTENODE_MEMORY_256M: 
    real_memory = 256; break;
  case BG_BP_COMPUTENODE_MEMORY_512M:
    real_memory = 512; break;
  case BG_BP_COMPUTENODE_MEMORY_1G:
    real_memory = 1024; break;
  case BG_BP_COMPUTENODE_MEMORY_2G:
    real_memory = 2048; break;
  case BG_BP_COMPUTENODE_MEMORY_4G:
    real_memory = 4096; break;
  case BG_BP_COMPUTENODE_MEMORY_NAV:
    real_memory = -1; break;
  default:
    real_memory = -1;
  }
  return real_memory;
}
main(int argc, char *argv[]) {
	LL_element *queryObject;
	LL_element *machine;
	LL_element *bp;
	LL_element  *node_card;
	char *BP_id;
	char *BP_current_partition;
	char *node_card_id;
	char *node_card_current_partition;
	int *BP_size;
	int BP_current_partition_state;
	int BP_node_card_count;
	int BP_Cnode_memory;
	int node_card_state;
	int node_card_quarter;
	int obj_count;
	int err_code;
	int rc;
	int state;
	int count;
	int boolean;
	int node_card_current_partition_state;
	const char *partid;
	const char *partstate;

	// Initialize the query blue gene
	queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		exit(1);
	}

	// Set the query request for all base partitions
	char *bp_list[2];
	bp_list[0] = "All";
	bp_list[1] = NULL;

	rc = ll_set_request(queryObject, QUERY_BG_BASE_PARTITION, bp_list, ALL_DATA);
	if (rc) {
		printf("Query MACHINES: ll_set_request() return code = %d\n",
		       rc);
		exit(1);
	}

	// Get the blue gene machine object. Only one machine object is returned
	// The blue gene machine object will only contain data for the base
	// partitions.
	err_code = 0;
	obj_count = 0;
	machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from blue gene machine
	rc = ll_get_data(machine, LL_BgMachineBPSize, &BP_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineBPSize rc = %d\n",rc);
		exit(1);
	}
	printf("Machine BP BP_size = %dx%dx%d\n",BP_size[0], BP_size[1], BP_size[2]);
	free(BP_size);

	//
	// Base Partitions
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstBP, &bp);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstBP rc = %d\n",rc);
		exit(1);
	}

	while(bp != NULL) {
		rc = ll_get_data(bp, LL_BgBPId, &BP_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPLocation, &BP_size);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPLocation rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPSubDividedBusy, &boolean);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPSubDividedBusy rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPCurrentPartition, &BP_current_partition);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPCurrentPartition rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPCurrentPartitionState, &BP_current_partition_state);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPCurrentPartitionState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPNodeCardCount, &BP_node_card_count);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPNodeCardCount rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPCnodeMemory, &BP_Cnode_memory);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPCnodeMemory rc = %d\n",rc);
			exit(1);
		}

		if (strlen(BP_current_partition) == 0) {
			partid = "NONE";
			partstate = "NONE";
		} else {
			partid = BP_current_partition;
			partstate = partitionStateString(BP_current_partition_state);
		}
		printf("\nBase Partition: id=%s c-node memory=%dM state=%-7.7s loc=%dx%dx%d sdb=%s curPar=%s curPState=%s Node_Card_Count=%d\n",
		       BP_id, get_real_memory(BP_Cnode_memory), bpStateString(state), BP_size[0], BP_size[1], BP_size[2],
		       boolean?"True":"False", partid, partstate, BP_node_card_count);
		free(BP_size);
		free(BP_id);
		free(BP_current_partition);

		//
		// Base Partition's Node Card
		//
		rc = ll_get_data(bp, LL_BgBPGetFirstNodeCard, &node_card);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPGetFirstNodecard rc = %d\n",rc);
			exit(1);
		}

		while(node_card != NULL) {
			rc = ll_get_data(node_card, LL_BgNodeCardId, &node_card_id);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardId rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardState, &node_card_state);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardState rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardQuarter, &node_card_quarter);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardQuarter rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardCurrentPartition, &node_card_current_partition);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardCurrentPartition rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(node_card, LL_BgNodeCardCurrentPartitionState, &node_card_current_partition_state);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeCardCurrentPartitionState rc = %d\n",rc);
				exit(1);
			}

			if (strlen(node_card_current_partition) == 0) {
				partid = "NONE";
				partstate = "NONE";
			} else {
				partid = node_card_current_partition;
				partstate = partitionStateString(node_card_current_partition_state);
			}
			printf("\tNode Card Id=%s State=%s Quarter=%s curPar=%-13.13s curPState=%s\n",
			       node_card_id,nodeCardState(node_card_state),nodeCardQuarter(node_card_quarter),
			       partid, partstate);
			free(node_card_id);
			free(node_card_current_partition);

			rc = ll_get_data(bp, LL_BgBPGetNextNodeCard, &node_card);
			if (rc) {
				printf("Error: ll_get_data LL_BgGetNextNodecard rc = %d\n",rc);
				exit(1);
			}

		}

		rc = ll_get_data(machine, LL_BgMachineGetNextBP, &bp);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextBP rc = %d\n",rc);
			exit(1);
		}
	}


	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);

}

const char* bpStateString(int state) {
	char* str_state;
	switch ( state ) {
	case BG_BP_UP:
		str_state = "UP";
		break;
	case BG_BP_DOWN:
		str_state = "DOWN";
		break;
	case BG_BP_MISSING:
		str_state = "MISSING";
		break;
	case BG_BP_ERROR:
		str_state = "ERROR";
		break;
	case BG_BP_NAV:
		str_state = "NAV";
		break;
	default:
		str_state = "<unknown>";
		break;
	}
	return str_state;
}

const char* partitionStateString(int state) {
	char* str_state;
	switch ( state ) {
	case BG_PARTITION_FREE:
		str_state = "FREE";
		break;
	case BG_PARTITION_CONFIGURING:
		str_state = "CFG";
		break;
	case BG_PARTITION_READY:
		str_state = "READY";
		break;
	case BG_PARTITION_BUSY:
		str_state = "BUSY";
		break;
	case BG_PARTITION_DEALLOCATING:
		str_state = "DEALC";
		break;
	case BG_PARTITION_ERROR:
		str_state = "ERROR";
		break;
	case BG_PARTITION_NAV:
		str_state = "NAV";
		break;
	default:
		str_state = "<unknown>";
		break;
	}
	return str_state;
}

const char* nodeCardState(int st) {
	char* str_state;
	switch ( st ) {
	case BG_NODE_CARD_UP:
		str_state = "UP";
		break;
	case BG_NODE_CARD_DOWN:
		str_state = "DOWN";
		break;
	case BG_NODE_CARD_MISSING:
		str_state = "MISSING";
		break;
	case BG_NODE_CARD_ERROR:
		str_state = "ERROR";
		break;
	case BG_NODE_CARD_NAV:
		str_state = "NAV";
		break;
	default:
		str_state = "<unknown>";
		break;
	}
	return str_state;
}

const char* nodeCardQuarter(int qt) {
	char* str_quarter;
	switch ( qt ) {
	case BG_QUARTER_Q1:
		str_quarter = "Q1";
		break;
	case BG_QUARTER_Q2:
		str_quarter = "Q2";
		break;
	case BG_QUARTER_Q3:
		str_quarter = "Q3";
		break;
	case BG_QUARTER_Q4:
		str_quarter = "Q4";
		break;
	case BG_QUARTER_Q_NAV:
		str_quarter = "NAV";
		break;
	default:
		str_quarter = "<unknown>";
		break;
	}
	return str_quarter;
}

const char* nodeMode(int m) {
	char* str_mode;
	switch ( m ) {
	case COPROCESSOR:
		str_mode = "COPROCESSOR";
		break;
	case VIRTUAL_NODE:
		str_mode = "VIRTUAL_NODE";
		break;
	default:
		str_mode = "<unknown>";
		break;
	}
	return str_mode;
}
